import zdf
import numpy as np
import matplotlib.pyplot as plt

np.set_printoptions(threshold=np.nan)

def create_boxplot(version="omp-atomic-1", t="lwfa"):
    if version in ("omp-atomic-1", "omp-atomic-2", "omp-reduction-1", "omp-reduction-2"):
        with open("../" + version + "/" + t + "-times.txt") as f:
            lines = f.readlines()
            lines = [x.strip('\n') for x in lines]
    group = {}
    num_threads = 0
    for line in lines:
        if "THREADS" in line and line not in group:
            num_threads = line.lower()
            group[num_threads] = []
        else:
            group[num_threads].append(float(line))
    
    box_plot_data = [group[x] for x in group]
    box_plot_labels = [x for x in group]
    plt.title("Version " + version)
    plt.boxplot(box_plot_data, vert=0, patch_artist=True, labels=box_plot_labels)
    plt.xlabel("time (s)")
    plt.show()

def get_files(version="serial", t="lwfa"):
    x = []
    y = []
    z = []
    if t == "lwfa":
        last_file = "001450"
    elif t == "weibel":
        last_file = "000500"
    if version in ("serial", "omp-atomic-1", "omp-atomic-2", "omp-reduction-1", "omp-reduction-2"):
        (x, sinfo) = zdf.read("../" + version + "/em2d/test-" + t + "/EMF/B1-" + last_file + ".zdf")
        (y, sinfo) = zdf.read("../" + version + "/em2d/test-" + t + "/EMF/B2-" + last_file + ".zdf")
        (z, sinfo) = zdf.read("../" + version + "/em2d/test-" + t + "/EMF/B3-" + last_file + ".zdf")
    return (x, y, z)

def get_average_error(version="omp-atomic-1", t="lwfa"):
    (x_s, y_s, z_s) = get_files("serial", t)
    size = np.size(x_s)
    if version in ("omp-atomic-1", "omp-atomic-2", "omp-reduction-1", "omp-reduction-2"):
        (x_p, y_p, z_p) = get_files(version, t)
        x = np.sum((x_s - x_p)**2) / size
        y = np.sum((y_s - y_p)**2) / size
        z = np.sum((z_s - z_p)**2) / size
        return (x + y + z) / 3
    else:
        print ("Invalid version")
        return -1

def show_lwfa(version="serial", diff="no"):
    (x, y, z) = get_files(version, t="lwfa")
    if diff == "yes":
        (x_s, y_s, z_s) = get_files("serial", t="lwfa")
    Bperp = np.sqrt( x**2 + y**2 ) if diff == "no" else np.sqrt( (x_s - x)**2 + (y_s - y)**2 )

    range = [[0,40.0],[0,51.2]]

    plt.imshow( Bperp, interpolation = 'bilinear', origin = 'lower',
            extent = ( range[0][0], range[0][1], range[1][0], range[1][1] ),
            aspect = 'auto', cmap = 'jet')

    plt.colorbar().set_label('Magnetic Field')
    plt.xlabel("$x_1$")
    plt.ylabel("$x_2$")
    plt.title("Magnetic Field\n")
    plt.show()

def show_weibel(version="serial", diff="no"):
    (x, y, z) = get_files(version, t="weibel")
    if diff == "yes":
        (x_s, y_s, z_s) = get_files("serial", t="weibel")
    Bperp = np.sqrt( x**2 + y**2 ) if diff == "no" else np.sqrt( (x_s - x)**2 + (y_s - y)**2 )

    range = [[0,51.2],[0,51.2]]

    plt.imshow( Bperp, interpolation = 'bilinear', origin = 'lower',
            extent = ( range[0][0], range[0][1], range[1][0], range[1][1] ),
            aspect = 'auto', cmap = 'jet')

    plt.colorbar().set_label('Magnetic Field')
    plt.xlabel("$x_1$")
    plt.ylabel("$x_2$")
    plt.title("Magnetic Field\n")
    plt.show()
